from huawei_lte_api.ApiGroup import ApiGroup
from huawei_lte_api.Session import GetResponseType


class MLog(ApiGroup):
    def mobile_logger(self) -> GetResponseType:
        """
        Endpoint found by reverse engineering B310s-22 firmware, unknown usage
        :return:
        """
        return self._session.get('mlog/mobile-logger')



from huawei_lte_api.ApiGroup import ApiGroup
from huawei_lte_api.Session import GetResponseType


class Ntwk(ApiGroup):

    def lan_upnp_portmapping(self) -> GetResponseType:
        return self._session.get('ntwk/lan_upnp_portmapping')

    def celllock(self) -> GetResponseType:
        return self._session.get('ntwk/celllock')



from huawei_lte_api.ApiGroup import ApiGroup
from huawei_lte_api.Session import GetResponseType


class Voice(ApiGroup):
    def featureswitch(self) -> GetResponseType:
        return self._session.get('voice/featureswitch')

    def sipaccount(self) -> GetResponseType:
        return self._session.get('voice/sipaccount')

    def sipadvance(self) -> GetResponseType:
        return self._session.get('voice/sipadvance')

    def sipserver(self) -> GetResponseType:
        return self._session.get('voice/sipserver')

    def speeddial(self) -> GetResponseType:
        return self._session.get('voice/speeddial')

    def functioncode(self) -> GetResponseType:
        return self._session.get('voice/functioncode')

    def voiceadvance(self) -> GetResponseType:
        return self._session.get('voice/voiceadvance')

    def voicebusy(self) -> GetResponseType:
        return self._session.get('voice/voicebusy')

    def codec(self) -> GetResponseType:
        """
        Endpoint found by reverse engineering B310s-22 firmware, unknown usage, probably not implemented by Huawei
        :return:
        """
        return self._session.get('voice/codec')



import os
from typing import BinaryIO

from huawei_lte_api.ApiGroup import ApiGroup
from huawei_lte_api.Session import SetResponseType


class FileManager(ApiGroup):
    def upload(self, uploadfile: BinaryIO, uploadfile_name: str) -> SetResponseType:
        """
        Uploads firmware update and triggers it
        :param uploadfile: filehandle on file to upload
        :param uploadfile_name: name of uploaded file
        :return: str
        """
        uploadfile_basename = os.path.basename(uploadfile_name)
        _, extension = os.path.splitext(uploadfile_basename)

        if extension.lower() not in ['.bin', '.zip']:
            raise Exception('Only *.bin or *.zip is allowed')

        return self._session.post_file('filemanager/upload', {
            'uploadfile': uploadfile,
        }, {
                                           'cur_path': 'OU:{}'.format(uploadfile_basename)
                                       })



from collections import OrderedDict
from typing import Union

from huawei_lte_api.ApiGroup import ApiGroup
from huawei_lte_api.Session import GetResponseType, SetResponseType
from huawei_lte_api.enums.net import NetworkModeEnum


class Net(ApiGroup):
    def current_plmn(self) -> GetResponseType:
        return self._session.get('net/current-plmn')

    def net_mode(self) -> GetResponseType:
        return self._session.get('net/net-mode')

    def set_net_mode(self, lteband: Union[str, int], networkband: Union[int, str], networkmode: Union[NetworkModeEnum, str]) -> SetResponseType:
        """
        :param lteband: bitmask of LTE band ints or'd together, where each band N is represented as 2**(N-1), as int,
            or hex str without leading '0x'. For example B1,B3,B20: 2**(1-1) | 2**(3-1) | 2**(20-1) = 0x80005.
            Use ALL for all or when not applicable (not 4G mode). All values or combinations of them may not be
            supported.
        :param networkband: bitmask of 3G network band ints or'd together, as int, or hex str without leading '0x'.
            See NetworkBandEnum, use ALL for all or when not applicable (not 3G mode). All values or combinations
            of them may not be supported.
        :param networkmode: network mode, see NetworkModeEnum; str supported for deprecated backwards compatibility
        :return:
        """
        return self._session.post_set('net/net-mode', OrderedDict((
            ('NetworkMode', networkmode if isinstance(networkmode, str) else networkmode.value),
            ('NetworkBand', networkband if isinstance(networkband, str) else hex(networkband)[2:]),
            ('LTEBand', lteband if isinstance(lteband, str) else hex(lteband)[2:]),
        )))

    def network(self) -> GetResponseType:
        return self._session.get('net/network')

    def register(self) -> GetResponseType:
        return self._session.get('net/register')

    def set_register(self, mode: str, plmn: str, rat: str) -> SetResponseType:
        """
        Sets network selection
        :param mode: "1": manual network selection, "0": auto
        :param plmn: Plmn code ("Numeric" value returned by net_mode_list()),
            "" for auto
        :param rat: "0": "2G", "2": "3G", "7": "4G" ("Rat" value returned by
            net_mode_list()), "" for auto
        :return: str
        """
        return self._session.post_set('net/register', OrderedDict((
            ('Mode', mode),
            ('Plmn', plmn),
            ('Rat', rat)
        )))

    def net_mode_list(self) -> GetResponseType:
        return self._session.get('net/net-mode-list')

    def plmn_list(self) -> GetResponseType:
        """
    !FIXME LOL DoS! no auth required :-D
    :return:
    """
        return self._session.get('net/plmn-list')

    def net_feature_switch(self) -> GetResponseType:
        return self._session.get('net/net-feature-switch')

    def cell_info(self) -> GetResponseType:
        return self._session.get('net/cell-info')

    def csps_state(self) -> GetResponseType:
        return self._session.get('net/csps_state')



from huawei_lte_api.ApiGroup import ApiGroup
from huawei_lte_api.Session import GetResponseType


class VSim(ApiGroup):
    def operateswitch_vsim(self) -> GetResponseType:
        return self._session.get('vsim/operateswitch-vsim')



from huawei_lte_api.Connection import Connection
from huawei_lte_api.api.Bluetooth import Bluetooth
from huawei_lte_api.api.Cradle import Cradle
from huawei_lte_api.api.Cwmp import Cwmp
from huawei_lte_api.api.DDns import DDns
from huawei_lte_api.api.Device import Device
from huawei_lte_api.api.Dhcp import Dhcp
from huawei_lte_api.api.Diagnosis import Diagnosis
from huawei_lte_api.api.DialUp import DialUp
from huawei_lte_api.api.FileManager import FileManager
from huawei_lte_api.api.Global import Global as Global_
from huawei_lte_api.api.Host import Host
from huawei_lte_api.api.Lan import Lan
from huawei_lte_api.api.Language import Language
from huawei_lte_api.api.Led import Led
from huawei_lte_api.api.Log import Log
from huawei_lte_api.api.MLog import MLog
from huawei_lte_api.api.Monitoring import Monitoring
from huawei_lte_api.api.Net import Net
from huawei_lte_api.api.Ntwk import Ntwk
from huawei_lte_api.api.OnlineUpdate import OnlineUpdate
from huawei_lte_api.api.Ota import Ota
from huawei_lte_api.api.Pb import Pb
from huawei_lte_api.api.Pin import Pin
from huawei_lte_api.api.Redirection import Redirection
from huawei_lte_api.api.SNtp import SNtp
from huawei_lte_api.api.SdCard import SdCard
from huawei_lte_api.api.Security import Security
from huawei_lte_api.api.Sms import Sms
from huawei_lte_api.api.Statistic import Statistic
from huawei_lte_api.api.Syslog import Syslog
from huawei_lte_api.api.Time import Time
from huawei_lte_api.api.TimeRule import TimeRule
from huawei_lte_api.api.UsbPrinter import UsbPrinter
from huawei_lte_api.api.UsbStorage import UsbStorage
from huawei_lte_api.api.User import User
from huawei_lte_api.api.VSim import VSim
from huawei_lte_api.api.Voice import Voice as Voice_
from huawei_lte_api.api.Vpn import Vpn
from huawei_lte_api.api.WLan import WLan
from huawei_lte_api.api.WebServer import WebServer
from huawei_lte_api.config.Device import Device as DeviceConfig
from huawei_lte_api.config.DeviceInformation import DeviceInformation
from huawei_lte_api.config.DialUp import DialUp as DialUpConfig
from huawei_lte_api.config.FastBoot import FastBoot
from huawei_lte_api.config.Firewall import Firewall
from huawei_lte_api.config.Global import Global
from huawei_lte_api.config.IPv6 import IPv6
from huawei_lte_api.config.Lan import Lan as LanConfig
from huawei_lte_api.config.Network import Network as NetworkConfig
from huawei_lte_api.config.Ota import Ota as OtaConfig
from huawei_lte_api.config.Pb import Pb as PbConfig
from huawei_lte_api.config.PcAssistant import PcAssistant
from huawei_lte_api.config.Pincode import Pincode as PincodeConfig
from huawei_lte_api.config.Sms import Sms as SmsConfig
from huawei_lte_api.config.Sntp import Sntp
from huawei_lte_api.config.Statistic import Statistic as ConfigStatistic
from huawei_lte_api.config.Stk import Stk
from huawei_lte_api.config.UPnp import UPnp
from huawei_lte_api.config.Update import Update
from huawei_lte_api.config.Ussd import Ussd
from huawei_lte_api.config.Voice import Voice
from huawei_lte_api.config.WebSd import WebSd
from huawei_lte_api.config.WebUICfg import WebUICfg
from huawei_lte_api.config.Wifi import Wifi as WifiConfig
from huawei_lte_api.usermanual.PublicSysResources import PublicSysResources


class Client:
    def __init__(self, connection: Connection):  # pylint: disable=too-many-statements
        self.monitoring = Monitoring(connection)
        self.security = Security(connection)
        self.webserver = WebServer(connection)
        self.global_ = Global_(connection)
        self.wlan = WLan(connection)
        self.cradle = Cradle(connection)
        self.pin = Pin(connection)
        self.config_dialup = DialUpConfig(connection)
        self.config_global = Global(connection)
        self.config_lan = LanConfig(connection)
        self.config_network = NetworkConfig(connection)
        self.config_pincode = PincodeConfig(connection)
        self.config_sms = SmsConfig(connection)
        self.config_voice = Voice(connection)
        self.config_wifi = WifiConfig(connection)
        self.config_pc_assistant = PcAssistant(connection)
        self.config_device_information = DeviceInformation(connection)
        self.config_web_ui_cfg = WebUICfg(connection)
        self.config_device = DeviceConfig(connection)
        self.config_fast_boot = FastBoot(connection)
        self.config_firewall = Firewall(connection)
        self.config_ipv6 = IPv6(connection)
        self.config_ota = OtaConfig(connection)
        self.config_pb = PbConfig(connection)
        self.config_sntp = Sntp(connection)
        self.config_statistic = ConfigStatistic(connection)
        self.config_stk = Stk(connection)
        self.config_update = Update(connection)
        self.config_u_pnp = UPnp(connection)
        self.config_ussd = Ussd(connection)
        self.config_web_sd = WebSd(connection)
        self.usermanual_public_sys_resources = PublicSysResources(connection)
        self.ota = Ota(connection)
        self.net = Net(connection)
        self.dial_up = DialUp(connection)
        self.sms = Sms(connection)
        self.redirection = Redirection(connection)
        self.v_sim = VSim(connection)
        self.file_manager = FileManager(connection)
        self.dhcp = Dhcp(connection)
        self.d_dns = DDns(connection)
        self.diagnosis = Diagnosis(connection)
        self.s_ntp = SNtp(connection)
        self.user = User(connection)
        self.device = Device(connection)
        self.online_update = OnlineUpdate(connection)
        self.log = Log(connection)
        self.time = Time(connection)
        self.sd_card = SdCard(connection)
        self.usb_storage = UsbStorage(connection)
        self.usb_printer = UsbPrinter(connection)
        self.vpn = Vpn(connection)
        self.ntwk = Ntwk(connection)
        self.pb = Pb(connection)
        self.host = Host(connection)
        self.language = Language(connection)
        self.syslog = Syslog(connection)
        self.voice = Voice_(connection)
        self.cwmp = Cwmp(connection)
        self.lan = Lan(connection)
        self.led = Led(connection)
        self.statistic = Statistic(connection)
        self.timerule = TimeRule(connection)
        self.bluetooth = Bluetooth(connection)
        self.mlog = MLog(connection)



from huawei_lte_api.ApiGroup import ApiGroup
from huawei_lte_api.Session import GetResponseType


class Vpn(ApiGroup):

    def feature_switch(self) -> GetResponseType:
        return self._session.get('vpn/feature-switch')

    def br_list(self) -> GetResponseType:
        return self._session.get('vpn/br_list')

    def ipsec_settings(self) -> GetResponseType:
        return self._session.get('vpn/ipsec_settings')

    def l2tp_settings(self) -> GetResponseType:
        return self._session.get('vpn/l2tp_settings')

    def pptp_settings(self) -> GetResponseType:
        return self._session.get('vpn/pptp_settings')

    def status(self) -> GetResponseType:
        """
        Endpoint found by reverse engineering B310s-22 firmware, unknown usage
        :return:
        """
        return self._session.get('vpn/status')



from huawei_lte_api.ApiGroup import ApiGroup
from huawei_lte_api.Session import GetResponseType


class Time(ApiGroup):

    def timeout(self) -> GetResponseType:
        return self._session.get('time/timeout')



from typing import Union

from binascii import hexlify
import math
import base64
from Cryptodome.Cipher import PKCS1_v1_5, PKCS1_OAEP
from Cryptodome.PublicKey.RSA import construct


class Tools:

    @staticmethod
    def enforce_list_response(data: dict, singular_key_name: str, plural_key_name: Union[str, None] = None) -> dict:
        """
        Make sure Hosts->Host is a list
        It may be returned as a single dict if only one is associated,
        as well as sometimes None.
        :return:
        """
        if not plural_key_name:
            plural_key_name = '{}s'.format(singular_key_name)
        if data.get(plural_key_name) is None:
            data[plural_key_name] = {}
        single_item = data[plural_key_name].setdefault(singular_key_name, [])
        if isinstance(single_item, dict):
            data[plural_key_name][singular_key_name] = [single_item]

        return data

    @staticmethod
    def rsa_encrypt(rsa_e: str, rsa_n: str, data: bytes, rsa_padding: int = 0) -> bytes:
        b64data = base64.b64encode(data)
        pubkey = construct((int(rsa_n, 16), int(rsa_e, 16)))

        cipher_module = {
            0: PKCS1_v1_5,
            1: PKCS1_OAEP
        }.get(rsa_padding)

        num = {
            0: 245,
            1: 214
        }.get(rsa_padding)

        if not cipher_module or not num:
            raise Exception('Unknown rsa_padding value {}'.format(rsa_padding))

        cipher = cipher_module.new(pubkey)  # type: ignore

        blocks = int(math.ceil(len(b64data) / float(num)))
        result_chunks = []
        for i in range(blocks):
            block = b64data[i * num:(i + 1) * num]
            result_chunks.append(cipher.encrypt(block))
        result = hexlify(b''.join(result_chunks))
        if (len(result) & 1) == 0:
            return result

        return b'0' + result



import base64
import hashlib
import time
from types import TracebackType
from typing import Union, Type, Optional
import requests

from huawei_lte_api.ApiGroup import ApiGroup
from huawei_lte_api.Session import GetResponseType, SetResponseType, Session
from huawei_lte_api.enums.client import ResponseEnum
from huawei_lte_api.enums.user import PasswordTypeEnum, LoginStateEnum, LoginErrorEnum
from huawei_lte_api.exceptions import ResponseErrorException, \
    LoginErrorAlreadyLoginException, \
    LoginErrorUsernamePasswordModifyException, \
    LoginErrorUsernamePasswordOverrunException, \
    LoginErrorUsernamePasswordWrongException, \
    LoginErrorUsernameWrongException, \
    LoginErrorPasswordWrongException, \
    ResponseErrorNotSupportedException


class UserSession:
    def __init__(self, session: Session, username: str, password: Union[str, None] = None):
        self.user = User(session)
        self.user.login(username, password, True)

    def close(self) -> None:
        self.user.logout()

    def __enter__(self) -> 'UserSession':
        return self

    def __exit__(self, exc_type: Optional[Type[BaseException]],
                 exc_value: Optional[BaseException],
                 traceback: Optional[TracebackType]) -> None:
        self.close()


class User(ApiGroup):

    def state_login(self) -> GetResponseType:
        return self._session.get('user/state-login')

    def _login(self, username: str, password: Union[str, None], password_type: PasswordTypeEnum = PasswordTypeEnum.BASE_64) -> bool:
        if not password:
            password_encoded = b''
        else:
            if password_type == PasswordTypeEnum.SHA256:
                concentrated = b''.join([
                    username.encode('UTF-8'),
                    base64.b64encode(hashlib.sha256(password.encode('UTF-8')).hexdigest().encode('ascii')),
                    self._session.request_verification_tokens[0].encode('UTF-8')
                ])
                password_encoded = base64.b64encode(hashlib.sha256(concentrated).hexdigest().encode('ascii'))
            else:
                password_encoded = base64.b64encode(password.encode('UTF-8'))

        try:
            result = self._session.post_set('user/login', {
                'Username': username,
                'Password': password_encoded.decode('UTF-8'),
                'password_type': password_type.value
            }, refresh_csrf=True)
        except ResponseErrorException as e:
            error_code_to_message = {
                LoginErrorEnum.USERNAME_WRONG.value: 'Username wrong',
                LoginErrorEnum.PASSWORD_WRONG.value: 'Password wrong',
                LoginErrorEnum.ALREADY_LOGIN.value: 'Already login',
                LoginErrorEnum.USERNAME_PWD_WRONG.value: 'Username and Password wrong',
                LoginErrorEnum.USERNAME_PWD_ORERRUN.value: 'Password overrun',
                LoginErrorEnum.USERNAME_PWD_MODIFY.value: 'Password modify',
            }

            error_code_to_exception = {
                LoginErrorEnum.USERNAME_WRONG.value: LoginErrorUsernameWrongException,
                LoginErrorEnum.PASSWORD_WRONG.value: LoginErrorPasswordWrongException,
                LoginErrorEnum.ALREADY_LOGIN.value: LoginErrorAlreadyLoginException,
                LoginErrorEnum.USERNAME_PWD_WRONG.value: LoginErrorUsernamePasswordWrongException,
                LoginErrorEnum.USERNAME_PWD_ORERRUN.value: LoginErrorUsernamePasswordOverrunException,
                LoginErrorEnum.USERNAME_PWD_MODIFY.value: LoginErrorUsernamePasswordModifyException,
            }

            message = error_code_to_message.get(e.code, 'Unknown')
            raise error_code_to_exception.get(e.code, ResponseErrorException)(
                '{}: {}'.format(e.code, message), e.code)

        return result == ResponseEnum.OK.value

    def login(self, username: str, password: Union[str, None], force_new_login: bool = False) -> bool:
        username = username if username else 'admin'
        tries = 5
        for i in range(tries):
            try:
                state_login = self.state_login()
            except requests.exceptions.ConnectionError:
                # Some models reportedly close the connection if we attempt to access login state too soon after
                # setting up the session etc. In that case, retry a few times. The error is reported to be
                # ConnectionError: (
                #     'Connection aborted.', RemoteDisconnected('Remote end closed connection without response'))
                if i == tries - 1:
                    raise
                time.sleep((i + 1) / 10)
            except ResponseErrorNotSupportedException:
                return True

        if LoginStateEnum(int(state_login['State'])) == LoginStateEnum.LOGGED_IN and not force_new_login:
            return True

        return self._login(username, password, PasswordTypeEnum(int(state_login['password_type'])))

    def logout(self) -> SetResponseType:
        return self._session.post_set('user/logout', {
            'Logout': 1
        })

    def remind(self) -> GetResponseType:
        return self._session.get('user/remind')

    def password(self) -> GetResponseType:
        return self._session.get('user/password')

    def pwd(self) -> GetResponseType:
        return self._session.get('user/pwd')

    def set_remind(self, remind_state: str) -> SetResponseType:
        return self._session.post_set('user/remind', {
            'remindstate': remind_state
        })

    def authentication_login(self) -> GetResponseType:
        return self._session.get('user/authentication_login')

    def challenge_login(self) -> GetResponseType:
        return self._session.get('user/challenge_login')

    def hilink_login(self) -> GetResponseType:
        return self._session.get('user/hilink_login')

    def history_login(self) -> GetResponseType:
        return self._session.get('user/history-login')

    def heartbeat(self) -> GetResponseType:
        return self._session.get('user/heartbeat')

    def web_feature_switch(self) -> GetResponseType:
        return self._session.get('user/web-feature-switch')

    def input_event(self) -> GetResponseType:
        """
       Endpoint found by reverse engineering B310s-22 firmware, unknown usage
       :return:
       """
        return self._session.get('user/input_event')

    def screen_state(self) -> GetResponseType:
        """
       Endpoint found by reverse engineering B310s-22 firmware, unknown usage
       :return:
       """
        return self._session.get('user/screen_state')

    def session(self) -> GetResponseType:
        """
        Endpoint found by reverse engineering B310s-22 firmware, unknown usage
        :return:
        """
        return self._session.get('user/session')



import warnings
from types import TracebackType
from typing import Optional, Tuple, Union, Type
from urllib.parse import urlparse

import requests

from huawei_lte_api.Session import Session
from huawei_lte_api.api.User import UserSession


class Connection(Session):
    def __init__(self,
                 url: str,
                 username: Optional[str] = None,
                 password: Optional[str] = None,
                 login_on_demand: bool = False,
                 timeout: Union[float, Tuple[float, float], None] = None,
                 requests_session: Optional[requests.Session] = None
                 ):
        parsed_url = urlparse(url)

        # User login code
        username = username if username else parsed_url.username
        password = password if password else parsed_url.password

        super().__init__(url, timeout=timeout, requests_session=requests_session)
        self.user_session = UserSession(self, username, password) if username else None

        if login_on_demand:
            warnings.warn(
                "login_on_demand is deprecated, and has no effect, please remove this parameter from your code! if  will get removed in next minor release.",
                DeprecationWarning
            )

    def close(self) -> None:
        if self.user_session:
            self.user_session.close()
        super().close()

    def __enter__(self) -> 'Connection':
        return self

    def __exit__(self, exc_type: Optional[Type[BaseException]],
                 exc_value: Optional[BaseException],
                 traceback: Optional[TracebackType]) -> None:
        self.close()



from huawei_lte_api.ApiGroup import ApiGroup
from huawei_lte_api.Session import GetResponseType


class Led(ApiGroup):
    def nightmode(self) -> GetResponseType:
        return self._session.get('led/nightmode')



from huawei_lte_api.ApiGroup import ApiGroup
from huawei_lte_api.Session import GetResponseType


class PublicSysResources(ApiGroup):
    def config(self) -> GetResponseType:
        return self._session.get('public_sys-resources/config.xml', prefix='usermanual')



from huawei_lte_api.ApiGroup import ApiGroup
from huawei_lte_api.Session import GetResponseType


class Log(ApiGroup):

    def loginfo(self) -> GetResponseType:
        return self._session.get('log/loginfo')



from huawei_lte_api.ApiGroup import ApiGroup
from huawei_lte_api.Session import GetResponseType


class DDns(ApiGroup):
    def get_ddns_list(self) -> GetResponseType:
        return self._session.get('ddns/ddns-list')

    def get_status(self) -> GetResponseType:
        return self._session.get('ddns/status')



from huawei_lte_api.ApiGroup import ApiGroup
from huawei_lte_api.Session import GetResponseType


class DialUp(ApiGroup):
    def config(self) -> GetResponseType:
        return self._session.get('dialup/config.xml', prefix='config')

    def connectmode(self) -> GetResponseType:
        return self._session.get('dialup/connectmode.xml', prefix='config')

    def profileswitch(self) -> GetResponseType:
        return self._session.get('dialup/profileswitch.xml', prefix='config')

    def lmt_auto_mode_disconnect(self) -> GetResponseType:
        return self._session.get('dialup/lmt_auto_mode_disconnect.xml', prefix='config')



from collections import OrderedDict

from huawei_lte_api.ApiGroup import ApiGroup
from huawei_lte_api.Session import GetResponseType, SetResponseType
from huawei_lte_api.enums.wlan import AuthModeEnum, WepEncryptModeEnum, WpaEncryptModeEnum
from huawei_lte_api.Tools import Tools


def _to_multi_basic_settings_ssid_body(ssid: dict) -> dict:
    return {
        'Index': ssid['Index'],
        'WifiBroadcast': ssid['WifiBroadcast'],
        'wifiguestofftime': ssid['wifiguestofftime'],
        'WifiAuthmode': ssid['WifiAuthmode'],
        'ID': ssid['ID'],
        'WifiEnable': ssid['WifiEnable'],
        'wifiisguestnetwork': ssid['wifiisguestnetwork'],
        'WifiMac': ssid['WifiMac'],
        'WifiSsid': ssid['WifiSsid'],
        'WifiRadiusKey': ssid['WifiRadiusKey'],
        'WifiWpaencryptionmodes': ssid['WifiWpaencryptionmodes'],
        'WifiWepKeyIndex': ssid['WifiWepKeyIndex'],
    }


def _set_wifi_enable(ssid: dict, status: bool) -> dict:
    if ssid['wifiisguestnetwork'] == '0':
        return ssid

    if status:
        wifiEnable = '1'
    else:
        wifiEnable = '0'

    ssid['WifiEnable'] = wifiEnable
    return ssid


class WLan(ApiGroup):
    def wifi_feature_switch(self) -> GetResponseType:
        return self._session.get('wlan/wifi-feature-switch')

    def station_information(self) -> GetResponseType:
        return self._session.get('wlan/station-information')

    def basic_settings(self) -> GetResponseType:
        return self._session.get('wlan/basic-settings')

    def set_basic_settings(self, ssid: str, hide: bool = False, wifi_restart: bool = False) -> SetResponseType:
        return self._session.post_set('wlan/basic-settings', OrderedDict((
            ('WifiSsid', ssid),
            ('WifiHide', hide),
            ('WifiRestart', int(wifi_restart))
        )))

    def security_settings(self) -> GetResponseType:
        return self._session.get('wlan/security-settings')

    def set_security_settings(self,
                              wpa_psk: str,
                              wep_key: str = '',
                              wpa_encryption_mode: WpaEncryptModeEnum = WpaEncryptModeEnum.MIX,
                              wep_encryption_mode: WepEncryptModeEnum = WepEncryptModeEnum.WEP128,
                              auth_mode: AuthModeEnum = AuthModeEnum.AUTO,
                              wifi_restart: bool = True
                              ) -> SetResponseType:
        return self._session.post_set('wlan/security-settings', OrderedDict((
            ('WifiAuthmode', auth_mode.value),
            ('WifiWepKey1', wep_key),
            ('WifiWpaencryptionmodes', wpa_encryption_mode.value),
            ('WifiBasicencryptionmodes', wep_encryption_mode.value),
            ('WifiWpapsk', wpa_psk),
            ('WifiRestart', int(wifi_restart))
        )))

    def multi_security_settings(self) -> GetResponseType:
        return self._session.get('wlan/multi-security-settings')

    def multi_security_settings_ex(self) -> GetResponseType:
        return self._session.get('wlan/multi-security-settings-ex')

    def multi_basic_settings(self) -> GetResponseType:
        return self._session.get('wlan/multi-basic-settings')

    def set_multi_basic_settings(self, clients: list) -> SetResponseType:
        """

   :param clients: list of dicts with format {'wifihostname': hostname,'WifiMacFilterMac': mac}
   """
        return self._session.post_set('wlan/multi-basic-settings', {
            'Ssids': {
                'Ssid': clients
            },
            'WifiRestart': 1
        })

    def host_list(self) -> GetResponseType:
        hosts = self._session.get('wlan/host-list')
        # Make sure Hosts->Host is a list
        # It may be returned as a single dict if only one is associated,
        # as well as sometimes None.
        return Tools.enforce_list_response(hosts, 'Host')

    def handover_setting(self) -> GetResponseType:
        return self._session.get('wlan/handover-setting')

    def set_handover_setting(self, handover: int) -> SetResponseType:
        """
        G3_PREFER = 0
        WIFI_PREFER = 2
        :param handover:
        """
        return self._session.post_set('wlan/handover-setting', {
            'Handover': handover
        })

    def multi_switch_settings(self) -> GetResponseType:
        return self._session.get('wlan/multi-switch-settings')

    def multi_macfilter_settings(self) -> GetResponseType:
        return self._session.get('wlan/multi-macfilter-settings')

    def set_multi_macfilter_settings(self, clients: list) -> SetResponseType:
        """

        :param clients: list of dicts with format {'wifihostname': hostname,'WifiMacFilterMac': mac}
        """
        return self._session.post_set('wlan/multi-macfilter-settings', {
            'Ssids': {
                'Ssid': clients
            }
        })

    def multi_macfilter_settings_ex(self) -> GetResponseType:
        return self._session.get('wlan/multi-macfilter-settings-ex')

    def mac_filter(self) -> GetResponseType:
        return self._session.get('wlan/mac-filter')

    def set_mac_filter(self, hostname: str, mac: str) -> SetResponseType:
        return self._session.post_set('wlan/mac-filter', OrderedDict((
            ('wifihostname', hostname),
            ('WifiMacFilterMac', mac)
        )))

    def oled_showpassword(self) -> GetResponseType:
        return self._session.get('wlan/oled-showpassword')

    def wps(self) -> GetResponseType:
        return self._session.get('wlan/wps')

    def wps_appin(self) -> GetResponseType:
        return self._session.get('wlan/wps-appin')

    def wps_pbc(self) -> GetResponseType:
        return self._session.get('wlan/wps-pbc')

    def wps_switch(self) -> GetResponseType:
        return self._session.get('wlan/wps-switch')

    def status_switch_settings(self) -> GetResponseType:
        return self._session.get('wlan/status-switch-settings')

    def wifiprofile(self) -> GetResponseType:
        """
        Endpoint found by reverse engineering B310s-22 firmware, unknown usage, probably not implemented by Huawei
        :return:
        """
        return self._session.get('wlan/wifiprofile')

    def wififrequence(self) -> GetResponseType:
        """
        Endpoint found by reverse engineering B310s-22 firmware, unknown usage, probably not implemented by Huawei
        :return:
        """
        return self._session.get('wlan/wififrequence')

    def wifiscanresult(self) -> GetResponseType:
        """
        Endpoint found by reverse engineering B310s-22 firmware, unknown usage, probably not implemented by Huawei
        :return:
        """
        return self._session.get('wlan/wifiscanresult')

    def wifi_guest_network_switch(self, status: bool) -> SetResponseType:
        """
        Turn on/off wifi guest network
        :param status: True->on, False->off
        """
        multi_basic_settings = self.multi_basic_settings()
        ssids = map(_to_multi_basic_settings_ssid_body, multi_basic_settings['Ssids']['Ssid'])
        ssids = map(lambda ssid: _set_wifi_enable(ssid, status), ssids)
        return self.set_multi_basic_settings(list(ssids))



from huawei_lte_api.ApiGroup import ApiGroup
from huawei_lte_api.Session import GetResponseType


class Global(ApiGroup):
    def module_switch(self) -> GetResponseType:
        return self._session.get('global/module-switch')

    def storage_get_item(self) -> GetResponseType:
        """
        Endpoint found by reverse engineering B310s-22 firmware, unknown usage
        :return:
        """
        return self._session.get('global/storage-getitem')



import warnings
from typing import Any
from huawei_lte_api.Connection import Connection


class AuthorizedConnection(Connection):
    def __init__(self, *args: Any, **kwargs: Any):
        warnings.warn("AuthorizedConnection is deprecated, use Connection instead", DeprecationWarning)
        super().__init__(*args, **kwargs)



from huawei_lte_api.ApiGroup import ApiGroup
from huawei_lte_api.Session import GetResponseType, SetResponseType


class Syslog(ApiGroup):
    def querylog(self) -> GetResponseType:
        return self._session.get('syslog/querylog')

    def clear(self) -> SetResponseType:
        return self._session.post_set('syslog/processlog', {
            'command': 'clear',
        })



import enum


@enum.unique
class NetworkModeEnum(enum.Enum):
    MODE_AUTO = '00'
    MODE_2G_ONLY = '01'
    MODE_3G_ONLY = '02'
    MODE_4G_ONLY = '03'
    MODE_4G_3G_AUTO = '0302'


@enum.unique
class NetworkBandEnum(enum.IntEnum):
    BC0A = 0x01
    BC0B = 0x02
    BC1 = 0x04
    BC2 = 0x08
    BC3 = 0x10
    BC4 = 0x20
    BC5 = 0x40
    GSM1800 = 0x80
    GSM900 = 0x300
    BC6 = 0x400
    BC7 = 0x800
    BC8 = 0x1000
    BC9 = 0x2000
    BC10 = 0x4000
    BC11 = 0x8000
    GSM850 = 0x80000
    GSM1900 = 0x200000
    UMTS_B1_2100 = 0x400000
    UMTS_B2_1900 = 0x800000
    BC12 = 0x10000000
    BC13 = 0x20000000
    UMTS_B5_850 = 0x4000000
    BC14 = 0x80000000
    UMTS_B8_900 = 0x2000000000000

    ALL = 0x3fffffff
    """Use alone, do not 'or' this with others."""


@enum.unique
class LTEBandEnum(enum.IntEnum):
    """For other values besides ALL, see set_net_mode docs."""
    ALL = 0x7fffffffffffffff
    """Use alone, do not 'or' this with others."""



import logging
import re
import urllib.parse
from types import TracebackType
from typing import Any, Callable, Dict, List, Optional, Tuple, TypeVar, Union, Type, cast
from urllib.parse import urlparse, urlunparse

import requests
import xmltodict

from huawei_lte_api.enums.client import ResponseCodeEnum
from huawei_lte_api.exceptions import \
    ResponseErrorException, \
    ResponseErrorLoginRequiredException, \
    ResponseErrorNotSupportedException, \
    ResponseErrorSystemBusyException, \
    ResponseErrorLoginCsrfException
from huawei_lte_api.Tools import Tools

_LOGGER = logging.getLogger(__name__)

T = TypeVar("T")
GetResponseType = Dict[str, Any]
SetResponseType = str


def _try_or_reload_and_retry(fn: Callable[..., T]) -> Callable[..., T]:
    def wrapped(*args: Any, **kw: Any) -> T:
        try:
            return fn(*args, **kw)
        except ResponseErrorLoginCsrfException:
            args[0].reload()
            return fn(*args, **kw)

    return wrapped


class Session:
    encryption_key = None
    csrf_re = re.compile(r'name="csrf_token"\s+content="(\S+)"')
    request_verification_tokens = []  # type: List[str]

    def __init__(self,
                 url: str,
                 timeout: Union[float, Tuple[float, float], None] = None,
                 requests_session: Optional[requests.Session] = None
                 ):

        # Auth info embedded in the URL may reportedly cause problems, strip it
        parsed_url = urlparse(url)
        clear_url = urlunparse((
            parsed_url.scheme,
            parsed_url.netloc.rpartition("@")[-1],
            *parsed_url[2:]
        ))

        self.requests_session = requests_session if requests_session else requests.Session()

        if not clear_url.endswith('/'):
            clear_url += '/'
        self.url = clear_url
        self.timeout = timeout

        # Try catch to close session correctly when _initialize_csrf_tokens_and_session fails
        try:
            self._initialize_csrf_tokens_and_session()
        except Exception:
            self.requests_session.close()
            raise

    def reload(self) -> None:
        self._initialize_csrf_tokens_and_session()

    @staticmethod
    def _create_request_xml(data: Union[dict, list, int]) -> bytes:
        wrapped_in_request = {
            'request': data
        }

        return xmltodict.unparse(wrapped_in_request).encode('utf-8')

    @staticmethod
    def _process_response_xml(response: requests.Response) -> dict:
        # In some cases unsupported methods, e.g. in config namespace,
        # respond with a redirect to the home page, which may not
        # parse as XML (even though it's labeled XHTML). Try to detect
        # such cases, and return a generated "not supported" error
        # instead of letting the XML parse error pass through.
        xml = response.content
        try:
            return xmltodict.parse(xml, dict_constructor=dict) if xml else {}
        except:
            if response.history:
                return dict(error=dict(code=ResponseCodeEnum.ERROR_SYSTEM_NO_SUPPORT, message=''))
            raise

    @staticmethod
    def _check_response_status(data: dict) -> Union[dict, str]:
        error_code_to_message = {
            ResponseCodeEnum.ERROR_SYSTEM_BUSY.value: 'System busy',
            ResponseCodeEnum.ERROR_SYSTEM_NO_RIGHTS.value: 'No rights (needs login)',
            ResponseCodeEnum.ERROR_SYSTEM_NO_SUPPORT.value: 'No support',
            ResponseCodeEnum.ERROR_SYSTEM_UNKNOWN.value: 'Unknown',
            ResponseCodeEnum.ERROR_SYSTEM_CSRF.value: 'Session error'
        }

        error_code_to_exception = {
            ResponseCodeEnum.ERROR_SYSTEM_BUSY.value: ResponseErrorSystemBusyException,
            ResponseCodeEnum.ERROR_SYSTEM_NO_RIGHTS.value: ResponseErrorLoginRequiredException,
            ResponseCodeEnum.ERROR_SYSTEM_NO_SUPPORT.value: ResponseErrorNotSupportedException,
            ResponseCodeEnum.ERROR_SYSTEM_UNKNOWN.value: ResponseErrorException,
            ResponseCodeEnum.ERROR_SYSTEM_CSRF.value: ResponseErrorLoginCsrfException
        }
        if 'error' in data:
            error_code = int(data['error']['code'])
            if not data['error']['message']:
                message = error_code_to_message.get(error_code, 'Unknown')
            else:
                message = data['error']['message']
            raise error_code_to_exception.get(error_code, ResponseErrorException)(
                '{}: {}'.format(error_code, message),
                error_code
            )

        response = data['response'] if 'response' in data else data
        return response if response is not None else {}

    def _initialize_csrf_tokens_and_session(self) -> None:
        # Reset
        self.request_verification_tokens = []

        # Lets try to parse csrf_token from homepage html head meta[name="csrf_token"]
        response = self.requests_session.get(self.url, timeout=self.timeout)

        csrf_tokens = self.csrf_re.findall(response.content.decode('UTF-8'))
        if csrf_tokens:
            self.request_verification_tokens = csrf_tokens
        else:
            # If we did not find anything in HTML, lets try to ask API for it
            token = self._get_token()
            if token is not None:
                self.request_verification_tokens.append(token)

    def _build_final_url(self, endpoint: str, prefix: str = 'api') -> str:
        return urllib.parse.urljoin(self.url + '{}/'.format(prefix), endpoint)

    def _encrypt_data(self, data: bytes) -> bytes:
        rsa_padding = self._get_rsa_padding()
        pubkey_data = self._get_encryption_key()
        rsa_e = pubkey_data.get('encpubkeye')
        rsa_n = pubkey_data.get('encpubkeyn')
        if not rsa_n or not rsa_e:
            raise Exception('No pub key was found')
        return Tools.rsa_encrypt(rsa_e, rsa_n, data, rsa_padding)

    def post_get(self,
                 endpoint: str,
                 data: Union[dict, list, int, None] = None,
                 refresh_csrf: bool = False,
                 prefix: str = 'api',
                 is_encrypted: bool = False) \
            -> GetResponseType:
        return cast(
            GetResponseType,
            self._post(endpoint, data, refresh_csrf, prefix, is_encrypted)
        )

    def post_set(self,
                 endpoint: str,
                 data: Union[dict, list, int, None] = None,
                 refresh_csrf: bool = False,
                 prefix: str = 'api',
                 is_encrypted: bool = False) \
            -> SetResponseType:
        return cast(
            SetResponseType,
            self._post(endpoint, data, refresh_csrf, prefix, is_encrypted)
        )

    @_try_or_reload_and_retry
    def _post(self,
              endpoint: str,
              data: Union[dict, list, int, None] = None,
              refresh_csrf: bool = False,
              prefix: str = 'api',
              is_encrypted: bool = False) \
            -> Union[GetResponseType, SetResponseType]:

        headers = {
            'Content-Type': 'application/x-www-form-urlencoded; charset=UTF-8;enc' if is_encrypted else 'application/xml'
        }

        if is_encrypted:
            headers['encrypt_transmit'] = 'encrypt_transmit'

        if self.request_verification_tokens:
            if len(self.request_verification_tokens) > 1:
                headers['__RequestVerificationToken'] = self.request_verification_tokens.pop(0)
            else:
                headers['__RequestVerificationToken'] = self.request_verification_tokens[0]

        data_encoded = self._create_request_xml(data) if data else b''
        response = self.requests_session.post(
            self._build_final_url(endpoint, prefix),
            data=self._encrypt_data(data_encoded) if is_encrypted else data_encoded,
            headers=headers,
            timeout=self.timeout,
        )
        response.raise_for_status()

        response_data = cast(str, self._check_response_status(self._process_response_xml(response)))

        if refresh_csrf:
            self.request_verification_tokens = []

        if '__RequestVerificationTokenone' in response.headers:
            self.request_verification_tokens.append(response.headers['__RequestVerificationTokenone'])
            if '__RequestVerificationTokentwo' in response.headers:
                self.request_verification_tokens.append(response.headers['__RequestVerificationTokentwo'])
        elif '__RequestVerificationToken' in response.headers:
            self.request_verification_tokens.append(response.headers['__RequestVerificationToken'])
        else:
            _LOGGER.debug('Failed to get CSRF from POST response headers')

        return response_data

    def post_file(self,
                  endpoint: str,
                  files: dict,
                  data: Optional[dict] = None,
                  prefix: str = 'api',
                  ) -> str:

        if self.request_verification_tokens:
            if data is None:
                data = {}
            data['csrf_token'] = self.request_verification_tokens[0]

        response = self.requests_session.post(
            self._build_final_url(endpoint, prefix),
            files=files,
            data=data,
            timeout=self.timeout
        )
        response.raise_for_status()

        return response.content.decode('UTF-8').lower()

    @_try_or_reload_and_retry
    def get(self, endpoint: str, params: Optional[dict] = None, prefix: str = 'api') -> dict:
        headers = {}
        if len(self.request_verification_tokens) == 1:
            headers['__RequestVerificationToken'] = self.request_verification_tokens[0]

        response = self.requests_session.get(
            self._build_final_url(endpoint, prefix),
            params=params,
            headers=headers,
            timeout=self.timeout,
        )

        return cast(dict, self._check_response_status(self._process_response_xml(response)))

    def _get_token(self) -> Optional[str]:
        try:
            data = self.get('webserver/token')
            return data['token']
        except ResponseErrorNotSupportedException:
            try:
                data = self.get('webserver/SesTokInfo')
                return data['TokInfo']
            except ResponseErrorNotSupportedException:
                return None

    def _get_encryption_key(self) -> dict:
        if not self.encryption_key:
            self.encryption_key = self.get('webserver/publickey')
        return self.encryption_key

    def _get_rsa_padding(self) -> int:
        state_login = self.get('user/state-login')
        return int(state_login['rsapadingtype']) if 'rsapadingtype' in state_login else 0

    def close(self) -> None:
        self.requests_session.close()

    def __enter__(self) -> 'Session':
        return self

    def __exit__(self, exc_type: Optional[Type[BaseException]],
                 exc_value: Optional[BaseException],
                 traceback: Optional[TracebackType]) -> None:
        self.close()



from typing import Optional
from huawei_lte_api.ApiGroup import ApiGroup
from huawei_lte_api.Session import GetResponseType, SetResponseType
from huawei_lte_api.enums.dialup import AuthModeEnum, IpType


class DialUp(ApiGroup):
    def mobile_dataswitch(self) -> GetResponseType:
        """
        Get current LTE modem toggle state
        :return:
        """
        return self._session.get('dialup/mobile-dataswitch')

    def connection(self) -> GetResponseType:
        return self._session.get('dialup/connection')

    def dialup_feature_switch(self) -> GetResponseType:
        return self._session.get('dialup/dialup-feature-switch')

    def profiles(self) -> GetResponseType:
        return self._session.get('dialup/profiles')

    def auto_apn(self) -> GetResponseType:
        return self._session.get('dialup/auto-apn')

    def dial(self) -> SetResponseType:
        return self._session.post_set('dialup/dial', {
            'Action': 1
        })

    def set_mobile_dataswitch(self, dataswitch: int = 0) -> SetResponseType:
        """
        Toggle LTE modem state
        :param dataswitch: 0 to disable LTE modem, 1 to enable LTE modem
        """
        return self._session.post_set('dialup/mobile-dataswitch', {
            'dataswitch': dataswitch
        })

    def set_default_profile(self, default: int = 0) -> SetResponseType:
        return self._session.post_set('dialup/profiles', {
            'SetDefault': default,
            'Delete': 0,
            'Modify': 0
        }, is_encrypted=True)

    def delete_profile(self, index: int) -> SetResponseType:
        return self._session.post_set('dialup/profiles', {
            'SetDefault': 0,
            'Delete': index,
            'Modify': 0
        }, is_encrypted=True)

    def create_profile(self,
                       name: str,
                       username: Optional[str] = None,
                       password: Optional[str] = None,
                       apn: Optional[str] = None,
                       dialup_number: Optional[str] = None,
                       auth_mode: AuthModeEnum = AuthModeEnum.AUTO,
                       ip_type: IpType = IpType.IPV4_IPV6,
                       is_default: bool = False
                       ) -> SetResponseType:
        return self._session.post_set('dialup/profiles', {
            'SetDefault': 1 if is_default else 0,
            'Delete': 0,
            'Modify': 1,
            'Profile': {
                'Index': '',
                'IsValid': 1,
                'Name': name,
                'ApnIsStatic': 1 if apn else 0,
                'ApnName': apn,
                'DialupNum': dialup_number,
                'Username': username,
                'Password': password,
                'AuthMode': auth_mode.value,
                'IpIsStatic': '',
                'IpAddress': '',
                'DnsIsStatic': '',
                'PrimaryDns': '',
                'SecondaryDns': '',
                'ReadOnly': '0',
                'iptype': ip_type.value
            }
        }, is_encrypted=True)



from collections import OrderedDict

from huawei_lte_api.ApiGroup import ApiGroup
from huawei_lte_api.Session import GetResponseType, SetResponseType


class SdCard(ApiGroup):

    def dlna_setting(self) -> GetResponseType:
        return self._session.get('sdcard/dlna-setting')

    def set_dlna_setting(self, enabled: bool, share_all: bool,
                         share_path: str = '/') -> SetResponseType:
        return self._session.post_set('sdcard/dlna-setting', {
            'enabled': int(enabled),
            'sharepath': share_path,
            'shareallpath': int(share_all),
        })

    def sdcard(self) -> GetResponseType:
        return self._session.get('sdcard/sdcard')

    def sdcardsamba(self) -> GetResponseType:
        return self._session.get('sdcard/sdcardsamba')

    def set_sdcardsamba(self, enabled: bool,
                        server_name: str = 'homerouter.cpe',
                        server_description: str = 'samba server',
                        workgroup_name: str = 'WORKGROUP',
                        anonymous_access: bool = False,
                        printer_enabled: bool = True) -> SetResponseType:
        return self._session.post_set('sdcard/sdcardsamba', OrderedDict((
            ('enabled', int(enabled)),
            ('servername', server_name),
            ('serverdescription', server_description),
            ('workgroupname', workgroup_name),
            ('anonymousaccess', int(anonymous_access)),
            ('printerenable', int(printer_enabled)),
        )))

    def printerlist(self) -> GetResponseType:
        return self._session.get('sdcard/printerlist')

    def share_account(self) -> GetResponseType:
        return self._session.get('sdcard/share-account')

    def sdfile(self) -> GetResponseType:
        """
        Endpoint found by reverse engineering B310s-22 firmware, unknown usage
        :return:
        """
        return self._session.get('sdcard/sdfile')

    def fileupload(self) -> GetResponseType:
        """
        Endpoint found by reverse engineering B310s-22 firmware, unknown usage
        :return:
        """
        return self._session.get('sdcard/fileupload')

    def check_file_exist(self) -> GetResponseType:
        """
        Endpoint found by reverse engineering B310s-22 firmware, unknown usage
        :return:
        """
        return self._session.get('sdcard/Check_file_exist')

    def createdir(self) -> GetResponseType:
        """
        Endpoint found by reverse engineering B310s-22 firmware, unknown usage
        :return:
        """
        return self._session.get('sdcard/createdir')

    def deletefile(self) -> GetResponseType:
        """
        Endpoint found by reverse engineering B310s-22 firmware, unknown usage
        :return:
        """
        return self._session.get('sdcard/deletefile')



from huawei_lte_api.ApiGroup import ApiGroup
from huawei_lte_api.Session import GetResponseType, SetResponseType
from huawei_lte_api.enums.device import AntennaTypeEnum


class Device(ApiGroup):
    def information(self) -> GetResponseType:
        return self._session.get('device/information')

    def autorun_version(self) -> GetResponseType:
        return self._session.get('device/autorun-version')

    def device_feature_switch(self) -> GetResponseType:
        return self._session.get('device/device-feature-switch')

    def basic_information(self) -> GetResponseType:
        return self._session.get('device/basic_information')

    def basicinformation(self) -> GetResponseType:
        return self._session.get('device/basicinformation')

    def usb_tethering_switch(self) -> GetResponseType:
        return self._session.get('device/usb-tethering-switch')

    def boot_time(self) -> GetResponseType:
        return self._session.get('device/boot_time')

    def set_control(self, control: int = 4) -> SetResponseType:
        return self._session.post_set('device/control', {
            'Control': control
        })

    def signal(self) -> GetResponseType:
        return self._session.get('device/signal')

    def control(self, control: int) -> SetResponseType:
        return self._session.post_set('device/control', {
            'Control': control
        })

    def reboot(self) -> SetResponseType:
        return self.control(1)

    def antenna_status(self) -> GetResponseType:
        return self._session.get('device/antenna_status')

    def get_antenna_settings(self) -> GetResponseType:
        return self._session.get('device/antenna_settings')

    def set_antenna_settings(self, antenna_type: AntennaTypeEnum = AntennaTypeEnum.AUTO) -> SetResponseType:
        return self._session.post_set('device/antenna_settings', {
            'antenna_type': antenna_type.value
        })

    def antenna_type(self) -> GetResponseType:
        return self._session.get('device/antenna_type')

    def antenna_set_type(self) -> GetResponseType:
        return self._session.get('device/antenna_set_type')

    def logsetting(self) -> GetResponseType:
        """
        Endpoint found by reverse engineering B310s-22 firmware, unknown usage
        :return:
        """
        return self._session.get('device/logsetting')



from huawei_lte_api.ApiGroup import ApiGroup
from huawei_lte_api.Session import GetResponseType


class WebServer(ApiGroup):
    def publickey(self) -> GetResponseType:
        return self._session.get('webserver/publickey')

    def token(self) -> GetResponseType:
        return self._session.get('webserver/token')

    def white_list_switch(self) -> GetResponseType:
        return self._session.get('webserver/white_list_switch')

    def ses_tok_info(self) -> GetResponseType:
        """
        Get session token info
        ~Valid for: B310s-22
        :return: GetResponseType
        """
        return self._session.get('webserver/SesTokInfo')



from setuptools import setup, find_packages


def read_readme() -> str:
    with open('README.md', 'r', encoding='utf-8') as f:
        return f.read()


setup(
    name='huawei-lte-api',
    version='1.5.1',
    packages=find_packages(exclude=['tests', 'tests.*']),
    package_data={'huawei_lte_api': ['py.typed']},
    install_requires=[
        'requests',
        'xmltodict',
    ],
    tests_require=[
        'tox'
    ],
    url='https://github.com/Salamek/huawei-lte-api',
    license='LGPL-3.0 ',
    author='Adam Schubert',
    author_email='adam.schubert@sg1-game.net',
    description='API For huawei LAN/WAN LTE Modems',
    long_description=read_readme(),
    long_description_content_type='text/markdown',
    test_suite='tests',
    classifiers=[
        'Development Status :: 5 - Production/Stable',
        'Environment :: Console',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: GNU Lesser General Public License v3 (LGPLv3)',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.4',
        'Programming Language :: Python :: 3.5',
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: 3.7',
        'Programming Language :: Python :: 3.8',
        'Topic :: Software Development',
    ],
    python_requires='>=3.4',
    project_urls={
        'Release notes': 'https://github.com/Salamek/huawei-lte-api/releases',
    },
)



from huawei_lte_api.ApiGroup import ApiGroup
from huawei_lte_api.Session import GetResponseType
from huawei_lte_api.Tools import Tools


class Lan(ApiGroup):
    def host_info(self) -> GetResponseType:
        hosts = self._session.get('lan/HostInfo')
        # Make sure Hosts->Host is a list
        # It may be returned as a single dict if only one is associated,
        # as well as sometimes None.
        return Tools.enforce_list_response(hosts, 'Host')



from typing import Optional

from huawei_lte_api.ApiGroup import ApiGroup
from huawei_lte_api.Session import GetResponseType, SetResponseType


class Pin(ApiGroup):
    def status(self) -> GetResponseType:
        return self._session.get('pin/status')

    def simlock(self) -> GetResponseType:
        return self._session.get('pin/simlock')

    def save_pin(self) -> GetResponseType:
        return self._session.get('pin/save-pin')

    def operate(self, operate_type: int = 0, current_pin: Optional[int] = None,
                new_pin: Optional[int] = None, puk_code: Optional[int] = None) \
            -> SetResponseType:
        """
        Parameters
        ----------
        operate_type : int
            Operation type to perform (default is `0`).
            0 - verify PIN
            1 - enable PIN verification
            2 - disable PIN verification
            3 - set new PIN
            4 - use of the PUK code
        current_pin : int
            Current PIN number (default is `None`).
        new_pin : int
            New PIN number to set (default is `None`).
        puk_code : int
            PUK code to use in case it is required by the device (default is `None`).
        """
        return self._session.post_set('pin/operate', {
            'OperateType': operate_type,
            'CurrentPin': current_pin,
            'NewPin': new_pin,
            'PukCode': puk_code
        })



from huawei_lte_api.ApiGroup import ApiGroup
from huawei_lte_api.Session import GetResponseType


class Redirection(ApiGroup):
    def homepage(self) -> GetResponseType:
        return self._session.get('redirection/homepage')
